@@ -3,7 +3,7 @@ goana <- function(de,...) UseMethod("goana")
 goana.MArrayLM <- function(de, coef = ncol(de), geneid = rownames(de), FDR = 0.05, trend = FALSE, ...)
 #	Gene ontology analysis of DE genes from linear model fit
 #	Gordon Smyth and Yifang Hu
-#	Created 20 June 2014.  Last modified 1 May 2015.
+#	Created 20 June 2014.  Last modified 11 Sep 2022.
 {
 #	Avoid argument collision with default method
 	dots <- names(list(...))
@@ -35,15 +35,17 @@ goana.MArrayLM <- function(de, coef = ncol(de), geneid = rownames(de), FDR = 0.0
 		if(trend) {
 			covariate <- de$Amean
 			if(is.null(covariate)) stop("Amean not found in fit")
+		} else {
+			covariate <- NULL
 		}
 	} else {
 		if(is.numeric(trend)) {
-			if(length(trend) != ngenes) stop("If trend is numeric, then length must equal nrow(de)")
+			if(!identical(length(trend),ngenes)) stop("If trend is numeric, then length must equal nrow(de)")
 			covariate <- trend
 			trend <- TRUE
 		} else {
 			if(is.character(trend)) {
-				if(length(trend) != 1L) stop("If trend is character, then length must be 1")
+				if(!identical(length(trend),1L)) stop("If trend is character, then length must be 1")
 				covariate <- de$genes[[trend]]
 				if(is.null(covariate)) stop("Column ",trend," not found in de$genes")
 				trend <- TRUE
@@ -52,9 +54,21 @@ goana.MArrayLM <- function(de, coef = ncol(de), geneid = rownames(de), FDR = 0.0
 		}
 	}
 
+#	Check for NA gene IDs or covariate values
+	if(anyNA(universe) || anyNA(covariate)) {
+		isna <- is.na(universe)
+		if(all(isna)) stop("Gene IDs are all NA")
+		if(trend) {
+			isna[is.na(covariate)] <- TRUE
+			covariate <- covariate[!isna]
+		}
+		universe <- universe[!isna]
+		de <- de[!isna,]
+	}
+
 #	Check FDR
 	if(!is.numeric(FDR) | length(FDR) != 1) stop("FDR must be numeric and of length 1.")
-	if(FDR < 0 | FDR > 1) stop("FDR should be between 0 and 1.")
+	if(FDR < 0 || FDR > 1) stop("FDR should be between 0 and 1.")
 
 #	Get up and down DE genes
 	fdr.coef <- p.adjust(de$p.value[,coef], method = "BH")
@@ -63,21 +77,18 @@ goana.MArrayLM <- function(de, coef = ncol(de), geneid = rownames(de), FDR = 0.0
 	DEGenes <- list(Up=EG.DE.UP, Down=EG.DE.DN)
 
 #	If no DE genes, return data.frame with 0 rows
-	if(length(EG.DE.UP)==0 && length(EG.DE.DN)==0) {
+	if( identical(length(EG.DE.UP),0L) && identical(length(EG.DE.DN),0L) ) {
 		message("No DE genes")
 		return(data.frame())
 	}
 
-	if(trend)
-		goana(de=DEGenes, universe = universe, covariate=covariate, ...)
-	else
-		goana(de=DEGenes, universe = universe, ...)
+	goana(de=DEGenes, universe = universe, covariate=covariate, ...)
 }
 
-goana.default <- function(de, universe = NULL, species = "Hs", prior.prob = NULL, covariate=NULL, plot=FALSE, ...)
+goana.default <- function(de, universe = NULL, species = "Hs", null.prob = NULL, covariate=NULL, plot=FALSE, ...)
 #	Gene ontology analysis of DE genes
 #	Gordon Smyth and Yifang Hu
-#	Created 20 June 2014.  Last modified 19 May 2019.
+#	Created 20 June 2014.  Last modified 11 Sep 2022.
 {
 #	Get access to package of GO terms
 	suppressPackageStartupMessages(OK <- requireNamespace("GO.db",quietly=TRUE))
@@ -104,23 +115,39 @@ goana.default <- function(de, universe = NULL, species = "Hs", prior.prob = NULL
 		i <- !duplicated(GeneID.PathID[,c("gene_id", "go_id")])
 		GeneID.PathID <- GeneID.PathID[i, ]
 		universe <- unique(GeneID.PathID[,1])
-		prior.prob <- covariate <- NULL
+		if( !(is.null(null.prob) && is.null(covariate)) ) {
+			message("Ignoring covariate and null.prob because universe not set")
+			null.prob <- covariate <- NULL
+		}
 	} else {
 		universe <- as.character(universe)
 		lu <- length(universe)
-		if(!is.null(prior.prob) && length(prior.prob)!=lu) stop("universe and prior.prob must have same length")
+		if(!is.null(null.prob) && length(null.prob)!=lu) stop("universe and null.prob must have same length")
 		if(!is.null(covariate) && length(covariate)!=lu) stop("universe and covariate must have same length")
+		if(anyNA(universe) || anyNA(covariate) || anyNA(null.prob)) {
+			isna <- is.na(universe)
+			if(all(isna)) stop("Gene IDs are all NA")
+			if(!is.null(covariate)) {
+				isna[is.na(covariate)] <- TRUE
+				covariate <- covariate[!isna]
+			}
+			if(!is.null(null.prob)) {
+				isna[is.na(null.prob)] <- TRUE
+				null.prob <- null.prob[!isna]
+			}
+			universe <- universe[!isna]
+		}
 		if(anyDuplicated(universe)) {
 			i <- !duplicated(universe)
 			if(!is.null(covariate)) covariate <- covariate[i]
-			if(!is.null(prior.prob)) prior.prob <- prior.prob[i]
+			if(!is.null(null.prob)) null.prob <- null.prob[i]
 			universe <- universe[i]
 		}
 #		Make universe and set of all annotated genes agree
 		i <- (universe %in% AnnotationDbi::Lkeys(egGO2ALLEGS))
 		universe <- universe[i]
 		if(!is.null(covariate)) covariate <- covariate[i]
-		if(!is.null(prior.prob)) prior.prob <- prior.prob[i]
+		if(!is.null(null.prob)) null.prob <- null.prob[i]
 		AnnotationDbi::Lkeys(egGO2ALLEGS) <- universe
 #		Convert GO annotation to data.frame
 		GeneID.PathID <- AnnotationDbi::toTable(egGO2ALLEGS)[,c("gene_id","go_id","Ontology")]
@@ -163,42 +190,44 @@ goana.default <- function(de, universe = NULL, species = "Hs", prior.prob = NULL
 	if(sum(i)==0L) stop("Pathways do not overlap with universe")
 	GeneID.PathID <- GeneID.PathID[i,]
 
-#	Get prior.prob trend in DE with respect to the covariate, combining all de lists
+#	Get null.prob trend in DE with respect to the covariate, combining all de lists
 	if(!is.null(covariate)) {
-		if(!is.null(prior.prob)) message("prior.prob being recomputed from covariate")
+		if(!is.null(null.prob)) message("null.prob being recomputed from covariate")
 		covariate <- as.numeric(covariate)
 		isDE <- (universe %in% unlist(de))
-		o <- order(covariate)
-		prior.prob <- covariate
-		span <- approx(x=c(20,200),y=c(1,0.5),xout=sum(isDE),rule=2,ties=list("ordered",mean))$y
-		prior.prob[o] <- tricubeMovingAverage(isDE[o],span=span)
-		if(plot) barcodeplot(covariate, index=isDE, worm=TRUE, span.worm=span, main="DE status vs covariate")
+		nDE <- sum(isDE)
+		if(identical(nDE,0L)) {
+			message("No DE genes")
+			null.prob <- covariate <- NULL
+		} else {
+			null.prob <- goanaTrend(isDE,covariate,plot=plot)
+		}
 	}
 
 #	Overlap pathways with DE genes
 #	Create incidence matrix (X) of gene.pathway by DE sets
-	if(is.null(prior.prob)) {
+	if(is.null(null.prob)) {
 		X <- matrix(1,nrow(GeneID.PathID),nsets+1)
 		colnames(X) <- c("N",names(de))
 	} else {
-		names(prior.prob) <- universe
+		names(null.prob) <- universe
 		X <- matrix(1,nrow(GeneID.PathID),nsets+2)
-		X[,nsets+2] <- prior.prob[GeneID.PathID[,1]]
+		X[,nsets+2] <- null.prob[GeneID.PathID[,1]]
 		colnames(X) <- c("N",names(de),"PP")
 	}
 	for (s in 1:nsets) X[,s+1] <- (GeneID.PathID[,1] %in% de[[s]])
 
-#	Count #genes and #DE genes and sum prior.prob for each pathway
+#	Count #genes and #DE genes and sum null.prob for each pathway
 	S <- rowsum(X, group=GeneID.PathID[,2], reorder=FALSE)
 
 #	Overlap tests
 	PValue <- matrix(0,nrow=nrow(S),ncol=nsets)
 	colnames(PValue) <- paste("P", names(de), sep=".")
 	nde <- lengths(de, use.names=FALSE)
-	if(!is.null(prior.prob)) {
+	if(!is.null(null.prob)) {
 
 #		Probability ratio for each pathway vs rest of universe
-		SumPP <- sum(prior.prob)
+		SumPP <- sum(null.prob)
 		M2 <- NGenes-S[,"N"]
 		Odds <- S[,"PP"] / (SumPP-S[,"PP"]) * M2 / S[,"N"]
 
@@ -210,7 +239,7 @@ goana.default <- function(de, universe = NULL, species = "Hs", prior.prob = NULL
 			               BiasedUrn::dWNCHypergeo(S[i,1L+j], S[i,"N"], M2[i], nde[j], Odds[i])
 
 #		Remove sum of prob column, not needed for output
-		S <- S[,-ncol(S)]
+		S <- S[,-ncol(S),drop=FALSE]
 
 	} else {
 
@@ -230,10 +259,10 @@ goana.default <- function(de, universe = NULL, species = "Hs", prior.prob = NULL
 	Results
 }
 
-topGO <- function(results, ontology = c("BP", "CC", "MF"), sort = NULL, number = 20L, truncate.term=NULL)
+topGO <- function(results, ontology = c("BP", "CC", "MF"), sort = NULL, number = 20L, truncate.term = NULL, p.value = 1)
 #	Extract top GO terms from goana output 
 #	Gordon Smyth and Yifang Hu
-#	Created 20 June 2014. Last modified 23 June 2016.
+#	Created 20 June 2014. Last modified 22 Dec 2022.
 {
 #	Check results
 	if(!is.data.frame(results)) stop("results should be a data.frame.")
@@ -250,7 +279,7 @@ topGO <- function(results, ontology = c("BP", "CC", "MF"), sort = NULL, number =
 
 #	Check number
 	if(!is.numeric(number)) stop("number should be a positive integer")
-	if(number > dimres[1L]) number <- dimres[1]
+	if(number > dimres[1]) number <- dimres[1]
 	if(number < 1L) return(results[integer(0),])
 
 #	Number of gene lists for which results are reported
@@ -268,13 +297,17 @@ topGO <- function(results, ontology = c("BP", "CC", "MF"), sort = NULL, number =
 		if(!length(isort)) stop("sort column not found in results")
 	}
 
-#	Sort by minimum p-value for specified gene lists
+#	Filter by minimum p-value for specified gene lists
 	P.col <- 3L+nsets+isort
 	if(length(P.col)==1L) {
 		P <- results[,P.col]
 	} else {
 		P <- do.call("pmin",as.data.frame(results[,P.col,drop=FALSE]))
 	}
+	if(p.value < 1) number <- min(number, sum(P <= p.value))
+	if(number < 1L) return(results[integer(0),])
+
+#	Sort by minimum p-value
 	o <- order(P,results$N,results$Term)
 	tab <- results[o[1L:number],,drop=FALSE]
 
