@@ -532,14 +532,15 @@ normalizeMedianValues <- function(x)
 	t(t(x)/cmed)
 }
 
-normalizeCyclicLoess <- function(x, weights = NULL, span=0.7, iterations = 3, method="fast")
+normalizeCyclicLoess <- function(x, weights = NULL, span=0.7, adaptive.span=FALSE, iterations = 3, method="fast")
 #	Cyclic loess normalization of columns of matrix
-#	incorporating probes weights.
+#	incorporating probe weights.
 #	Yunshun (Andy) Chen and Gordon Smyth
-#	14 April 2010.  Last modified 24 Feb 2012.
+#	14 April 2010.  Last modified 14 June 2024.
 {
 	x <- as.matrix(x)
 	method <- match.arg(method, c("fast","affy","pairs"))
+	if(adaptive.span) span <- chooseLowessSpan(nrow(x),small.n=200,min.span=0.6)
 	n <- ncol(x)
 	if(method=="pairs") {
 		for (k in 1:iterations)
