@@ -1,8 +1,12 @@
-voom <- function(counts,design=NULL,lib.size=NULL,normalize.method="none",block=NULL,correlation=NULL,weights=NULL,span=0.5,plot=FALSE,save.plot=FALSE)
+voom <- function(
+	counts, design=NULL, lib.size=NULL, normalize.method="none",
+	block=NULL, correlation=NULL, weights=NULL,
+	span=0.5, adaptive.span=FALSE, plot=FALSE, save.plot=FALSE
+)
 #	Linear modelling of count data with mean-variance modelling at the observation level.
 #	Creates an EList object for entry to lmFit() etc in the limma pipeline.
 #	Gordon Smyth and Charity Law
-#	Created 22 June 2011.  Last modified 1 May 2021.
+#	Created 22 June 2011.  Last modified 14 June 2024.
 {
 	out <- list()
 
@@ -25,8 +29,8 @@ voom <- function(counts,design=NULL,lib.size=NULL,normalize.method="none",block=
 	}
 
 #	Check counts
-	n <- nrow(counts)
-	if(n < 2L) stop("Need at least two genes to fit a mean-variance trend")
+	ngenes <- nrow(counts)
+	if(ngenes < 2L) stop("Need at least two genes to fit a mean-variance trend")
 	m <- min(counts)
 	if(is.na(m)) stop("NA counts not allowed")
 	if(m < 0) stop("Negative counts not allowed")
@@ -41,6 +45,9 @@ voom <- function(counts,design=NULL,lib.size=NULL,normalize.method="none",block=
 #	Check lib.size
 	if(is.null(lib.size)) lib.size <- colSums(counts)
 
+#	Choose span based on the number of genes
+	if(adaptive.span) span <- chooseLowessSpan(ngenes, small.n=50, min.span=0.3, power=1/3)
+
 #	Fit linear model to log2-counts-per-million
 	y <- t(log2(t(counts+0.5)/(lib.size+1)*1e6))
 	y <- normalizeBetweenArrays(y,method=normalize.method)
@@ -111,8 +118,9 @@ voom <- function(counts,design=NULL,lib.size=NULL,normalize.method="none",block=
 		out$targets <- data.frame(lib.size=lib.size)
 	else
 		out$targets$lib.size <- lib.size
+	if(adaptive.span) out$span <- span
 	if(save.plot) {
-		out$voom.xy <- list(x=sx,y=sy,xlab="log2( count size + 0.5 )",ylab="Sqrt( standard deviation )")
+		out$voom.xy <- list(x=sx,y=sy,xlab="log2( count size + 0.5 )",ylab="Sqrt( standard deviation )",pch=16,cex=0.25)
 		out$voom.line <- l
 	}
 
